searchData={"items":[{"type":"module","title":"Attrs","doc":"","ref":"Attrs.html"},{"type":"function","title":"Attrs.get/3","doc":"","ref":"Attrs.html#get/3"},{"type":"function","title":"Attrs.has_key?/2","doc":"","ref":"Attrs.html#has_key?/2"},{"type":"function","title":"Attrs.merge/2","doc":"","ref":"Attrs.html#merge/2"},{"type":"function","title":"Attrs.normalize_keys/1","doc":"","ref":"Attrs.html#normalize_keys/1"},{"type":"function","title":"Attrs.put/3","doc":"","ref":"Attrs.html#put/3"},{"type":"function","title":"Attrs.stringify_keys/1","doc":"","ref":"Attrs.html#stringify_keys/1"},{"type":"extras","title":"Attrs","doc":"# Attrs\n\nUnifying atom and string key handling for user data (attrs maps) given to Ecto's cast function.","ref":"readme.html"},{"type":"extras","title":"Problem - Attrs","doc":"When working with Elixir's Phoenix Controllers or Absinthe, external data maps passed to `cast/4` must consist entirely of either atom keys or string keys, but cannot contain a mix of both types. This leads to tedious checks and potentially error-prone adjustments for data retrieval and insertion, since the key type must be accounted for each time:\n```elixir\nuser_id = attrs[:user_id] || attrs[\"user_id\"]\n```","ref":"readme.html#problem"},{"type":"extras","title":"Solution - Attrs","doc":"```elixir\nuser_id = Attrs.get(attrs, :user_id)\n```\n\nAttrs enables handling of maps irrespective of whether they contain atom or string keys. It eliminates the distinction between the two key types, simplifying code when working with external data maps passed to `cast/4`.\n\nWhy \"Attrs\"? In Phoenix Contexts, user parameters and external data are often labeled as attrs, hence the name of this package.","ref":"readme.html#solution"},{"type":"extras","title":"Attrs module - Attrs","doc":"### `Attrs.get(attrs, key, default)`\n\n```elixir\n# Regardless of whether the key is a string or an atom, the value can be retrieved\nAttrs.get(%{foo: 1}, :foo) == 1\nAttrs.get(%{\"foo\" => 1}, :foo) == 1\n\n# A default value may be provided in case the key is not found in the map\nAttrs.get(%{foo: 1}, :bar) == nil\nAttrs.get(%{foo: 1}, :bar, 2) == 2\n```\n\n### `Attrs.has_key?(attrs, key)`\n\n```elixir\nAttrs.has_key?(%{foo: 1}, :foo) == true\nAttrs.has_key?(%{\"foo\" => 1}, :foo) == true\n```\n\n### `Attrs.put(attrs, key, value)`\n\n```elixir\n# Preserve key type based on the first key returned from `Map.keys/1`.\nAttrs.put(%{\"foo\" => 1}, :bar, 2) == %{\"foo\" => 1, \"bar\" => 2}\nAttrs.put(%{foo: 1}, :bar, 2) == %{foo: 1, bar: 2}\n```\n\n### `Attrs.merge(attrs1, attrs2)`\n\n```elixir\n# Preserve key type based on the first key returned from `Map.keys/1`.\n# If the key of either input map is a string key, the merged map will use string keys.\n# If the keys of both input maps are atom keys, the merged map will use atom keys.\nAttrs.merge(%{\"foo\" => 1}, %{bar: 2}) == %{\"foo\" => 1, \"bar\" => 2}\nAttrs.merge(%{\"foo\" => 1}, %{\"bar\" => 2}) == %{\"foo\" => 1, \"bar\" => 2}\nAttrs.merge(%{foo: 1}, %{bar: 2}) == %{foo: 1, bar: 2}\n```\n\n### `Attrs.normalize_keys(attrs)`\n\n```elixir\n# When normalizing key types, if the input map contains a mix of atom and string keys, all keys are converted to strings\nAttrs.normalize_keys(%{\"foo\" => 1, bar: 2}) == %{\"foo\" => 1, \"bar\" => 2}\n\n# If all keys are already atoms or strings, they are left unchanged\nAttrs.normalize_keys(%{foo: 1, bar: 2}) == %{foo: 1, bar: 2}\nAttrs.normalize_keys(%{\"foo\" => 1, \"bar\" => 2}) == %{\"foo\" => 1, \"bar\" => 2}\n```\n\n### `Attrs.stringify_keys(attrs)`\n\n```elixir\nAttrs.stringify_keys(%{\"foo\" => 1, bar: 2}) == %{\"foo\" => 1, \"bar\" => 2}\nAttrs.stringify_keys(%{foo: 1, bar: 2}) == %{\"foo\" => 1, \"bar\" => 2}\nAttrs.stringify_keys(%{\"foo\" => 1, \"bar\" => 2}) == %{\"foo\" => 1, \"bar\" => 2}\n```","ref":"readme.html#attrs-module"},{"type":"extras","title":"Installation - Attrs","doc":"Add `attrs` for Elixir as a dependency in your `mix.exs` file:\n\n```elixir\ndef deps do\n  [\n    {:attrs, \"~> 0.5.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"HexDocs - Attrs","doc":"HexDocs documentation can be found at [https://hexdocs.pm/attrs](https://hexdocs.pm/attrs).","ref":"readme.html#hexdocs"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}